@@ -25,6 +25,7 @@
 #endif
 
 #else
+#include <algorithm>
 #include <sys/wait.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -120,6 +121,7 @@ VBOX_BASE::VBOX_BASE() : VBOX_JOB() {
     headless = true;
     vm_pid = 0;
     vboxsvc_pid = 0;
+
 #ifdef _WIN32
     vm_pid_handle = 0;
     vboxsvc_pid_handle = 0;
@@ -149,6 +151,7 @@ int VBOX_BASE::run(bool do_restore_snapshot) {
 
     retval = is_registered();
     if (ERR_TIMEOUT == retval) {
+        vboxlog_msg("Error: Timeout");
 
         return VBOXWRAPPER_ERR_RECOVERABLE;
 
@@ -156,19 +159,33 @@ int VBOX_BASE::run(bool do_restore_snapshot) {
 
         if (is_vm_machine_configuration_available()) {
             retval = register_vm();
-            if (retval) return retval;
+          
+            if (retval){
+
+                vboxlog_msg("Could not register");
+                return retval;
+            }
         } else {
             if (is_disk_image_registered()) {
                 // Handle the case where a previous instance of the same projects VM
                 // was already initialized for the current slot directory but aborted
                 // while the task was suspended and unloaded from memory.
                 retval = deregister_stale_vm();
-                if (retval) return retval;
+              
+                if (retval){
+
+                    vboxlog_msg("Could not deregister stale VM");
+                    return retval;
+                }
             }
             retval = create_vm();
-            if (retval) return retval;
-        }
 
+            if (retval){
+
+                vboxlog_msg("Could not create VM");
+                return retval;
+            }
+        }
     }
 
     // The user has requested that we exit after registering the VM, so return an
@@ -181,22 +198,33 @@ int VBOX_BASE::run(bool do_restore_snapshot) {
     vm_name = vm_master_name;
 
     // Check to see if the VM is already in a running state, if so, poweroff.
-    poll(false);
+    poll2(false);
     if (online) {
+        vboxlog_msg("VM was running");
         retval = poweroff();
-        if (retval) return ERR_NOT_EXITED;
+
+        if (retval){
+
+            vboxlog_msg("Could not stop running VM");
+            return ERR_NOT_EXITED;
+        }
     }
 
     // If our last checkpoint time is greater than 0, restore from the previously
     // saved snapshot
     if (do_restore_snapshot) {
         retval = restore_snapshot();
-        if (retval) return retval;
+
+        if (retval){
+
+            vboxlog_msg("Could not restore from snapshot");
+            return retval;
+        }
     }
 
     // Has BOINC signaled that we should quit?
     // Try to prevent starting the VM in an environment where we might be terminated any
-    // second.  This can happen if BOINC has been told to shutdown or the volunteer has 
+    // second.  This can happen if BOINC has been told to shutdown or the volunteer has
     // told BOINC to switch to a different project.
     //
     if (boinc_status.no_heartbeat || boinc_status.quit_request) {
@@ -205,7 +233,12 @@ int VBOX_BASE::run(bool do_restore_snapshot) {
 
     // Start the VM
     retval = start();
-    if (retval) return retval;
+
+    if (retval){
+
+        vboxlog_msg("Could not start ");
+        return retval;
+    }
 
     return 0;
 }
@@ -232,94 +265,260 @@ void VBOX_BASE::dump_hypervisor_logs(bool include_error_logs) {
     get_vm_exit_code(vm_exit_code);
 
     if (include_error_logs) {
-		dump_screenshot();
+        dump_screenshot();
         fprintf(
-            stderr,
-            "\n"
-            "    Hypervisor System Log:\n\n"
-            "%s\n"
-            "    VM Execution Log:\n\n"
-            "%s\n"
-            "    VM Startup Log:\n\n"
-            "%s\n"
-            "    VM Trace Log:\n\n"
-            "%s",
-            local_system_log.c_str(),
-            local_vm_log.c_str(),
-            local_startup_log.c_str(),
-            local_trace_log.c_str()
-        );
+                stderr,
+                "\n"
+                "    Hypervisor System Log:\n\n"
+                "%s\n"
+                "    VM Execution Log:\n\n"
+                "%s\n"
+                "    VM Startup Log:\n\n"
+                "%s\n"
+                "    VM Trace Log:\n\n"
+                "%s",
+                local_system_log.c_str(),
+                local_vm_log.c_str(),
+                local_startup_log.c_str(),
+                local_trace_log.c_str()
+               );
     }
 
     if (vm_exit_code) {
         fprintf(
-            stderr,
-            "\n"
-            "    VM Exit Code: %d (0x%x)\n\n",
-            (unsigned int)vm_exit_code,
-            (unsigned int)vm_exit_code
-        );
+                stderr,
+                "\n"
+                "    VM Exit Code: %d (0x%x)\n\n",
+                (unsigned int)vm_exit_code,
+                (unsigned int)vm_exit_code
+               );
     }
 }
 
+void VBOX_BASE::report_clean(bool unrecoverable_error, bool skip_cleanup, bool do_dump_hypervisor_logs,
+        int retval, std::string error_reason,
+        int vm_pid, int temp_delay, std::string temp_reason,
+        double current_cpu_time,
+        double last_checkpoint_cpu_time,
+        double fraction_done,
+        double bytes_sent,
+        double bytes_received) {
+
+
+    if (unrecoverable_error) {
+
+        // Attempt to cleanup the VM and exit.
+        if (!skip_cleanup) {
+            cleanup();
+        }
+
+        if (error_reason.size()) {
+            vboxlog_msg("\n%s", error_reason.c_str());
+        }
+
+        if (do_dump_hypervisor_logs) {
+            dump_hypervisor_logs(true);
+        }
+
+        boinc_finish(retval);
+    }
+    else {
+
+        // if the VM is already running notify BOINC about the process ID so it can
+        // clean up the environment.  We should be safe to run after that.
+        //
+        if (vm_pid) {
+            retval = boinc_report_app_status_aux(
+                    current_cpu_time,
+                    last_checkpoint_cpu_time,
+                    fraction_done,
+                    vm_pid,
+                    bytes_sent,
+                    bytes_received
+                    );
+        }
+
+        // Give the BOINC API time to report the pid to BOINC.
+        //
+        boinc_sleep(5.0);
+
+        if (error_reason.size()) {
+            vboxlog_msg("\n%s", error_reason.c_str());
+        }
+
+        if (do_dump_hypervisor_logs) {
+            dump_hypervisor_logs(true);
+        }
+
+        // Exit and let BOINC clean up the rest.
+        //
+        boinc_temporary_exit(temp_delay, temp_reason.c_str());
+    }
+}
+
+string VBOX_BASE::get_error(int num){
+
+    const char* args[] = {"   BOINC will be notified that it needs to clean up the environment.\n \
+        This is a temporary problem and so this job will be rescheduled for another time.\n",
+
+          "   NOTE: VM was already running.\n \
+              BOINC will be notified that it needs to clean up the environment.\n \
+              This might be a temporary problem and so this job will be rescheduled for another time.\n",
+
+          "   NOTE: VirtualBox has reported an improperly configured virtual machine. It was configured to require\n \
+              hardware acceleration for virtual machines, but your processor does not support the required feature.\n \
+              Please report this issue to the project so that it can be addresssed.\n \
+              Error Code: ERR_CPU_VM_EXTENSIONS_DISABLED\n",
+
+          "   VboxSvc crashed while attempting to restore the current snapshot.  This is a critical\n \
+              operation and this job cannot be recovered.\n",
+
+          "   NOTE: VM session lock error encountered.\n \
+              BOINC will be notified that it needs to clean up the environment.\n \
+              This might be a temporary problem and so this job will be rescheduled for another time.\n",
+
+          "   NOTE: BOINC has detected that your computer's processor supports hardware acceleration for\n \
+              virtual machines but the hypervisor failed to successfully launch with this feature enabled.\n \
+              This means that the hardware acceleration feature has been disabled in the computer's BIOS.\n \
+              Please enable this feature in your computer's BIOS.\n \
+              Intel calls it 'VT-x'\n \
+              AMD calls it 'AMD-V'\n \
+              More information can be found here: https://en.wikipedia.org/wiki/X86_virtualization\n \
+              Error Code: ERR_CPU_VM_EXTENSIONS_DISABLED\n",
+
+          "   NOTE: VirtualBox hypervisor reports that another hypervisor has locked the hardware acceleration\n \
+              for virtual machines feature in exclusive mode.\n",
+
+          "   NOTE: VirtualBox has failed to allocate enough memory to start the configured virtual machine.\n \
+              This might be a temporary problem and so this job will be rescheduled for another time.\n",
+
+          "   NOTE: VM failed to enter an online state within the timeout period.\n \
+              This might be a temporary problem and so this job will be rescheduled for another time.\n",
+
+          "VM environment needed to be cleaned up.",
+
+          "Foreign VM Hypervisor locked hardware acceleration features.",
+
+          "VM Hypervisor was unable to allocate enough memory to start VM.",
+
+          "VM Hypervisor failed to enter an online state in a timely fashion."
+    };
+
+    std::vector<std::string> v(args, args + 13);
+    return v[num];
+}
+
+string VBOX_BASE::read_vm_log(){
+
+    string line;
+    size_t line_pos;
+    size_t line_end;
+    size_t line_start;
+    string msg;
+    static string state = "poweredoff";
+    string virtualbox_vm_log;
+    virtualbox_vm_log = vm_master_name + "/Logs/VBox.log";
+
+    string console = "Console: Machine state changed to \'";
+
+    std::ifstream  src(virtualbox_vm_log.c_str(), std::ios::binary);
+
+    if ((src.is_open()) && (src.seekg(log_pointer)))
+    {
+
+        while (std::getline(src, line))
+        {
+            line_start = line.find(console);
+            if (line_start != string::npos) {
+
+                line_start += console.size();
+
+                line_end = line.find("\'", line_start);
+
+                msg = line.substr(line_start, line_end - line_start);
+
+                sanitize_format(msg);
+
+                state = msg;
+
+		std::transform(state.begin(), state.end(), state.begin(), ::tolower);
+            }
+
+            line_pos = line.find("Guest Log:");
+            if (line_pos != string::npos) {
+
+                msg = line.substr(line_pos, line.size() - line_pos);
+
+                sanitize_format(msg);
+                sanitize_output(msg);
+
+                vboxlog_msg(msg.c_str());
+
+            }
+            log_pointer = src.tellg();
+        }
+
+        return state;
+    }
+    else return "Error in parsing the log file";
+}
+
+
 // Dump any new guest log messages which are generated by applications running within
 // the guest VM.
 void VBOX_BASE::dump_vmguestlog_entries() {
     string local_vm_log;
     string line;
-    size_t eol_pos;
-    size_t eol_prev_pos;
     size_t line_pos;
     VBOX_TIMESTAMP current_timestamp;
     string msg;
+    string virtualbox_vm_log;
+    virtualbox_vm_log = vm_master_name + "/Logs/VBox.log";
+    int check;
 
-    get_vm_log(local_vm_log, true, 16*1024);
+    if (boinc_file_exists(virtualbox_vm_log.c_str())) {
 
-    eol_prev_pos = 0;
-    eol_pos = local_vm_log.find("\n");
-    while (eol_pos != string::npos) {
-        line = local_vm_log.substr(eol_prev_pos, eol_pos - eol_prev_pos);
+        std::ifstream  src(virtualbox_vm_log.c_str(), std::ios::in);
+        while (std::getline(src, line))
+        {
+            line_pos = line.find("Guest Log:");
+            if (line_pos != string::npos) {
+                check = sscanf(
+                        line.c_str(),
+                        "%d:%d:%d.%d",
+                        &current_timestamp.hours, &current_timestamp.minutes,
+                        &current_timestamp.seconds, &current_timestamp.milliseconds
+                        );
 
-        line_pos = line.find("Guest Log:");
-        if (line_pos != string::npos) {
-            sscanf(
-                line.c_str(),
-                "%d:%d:%d.%d",
-                &current_timestamp.hours, &current_timestamp.minutes,
-                &current_timestamp.seconds, &current_timestamp.milliseconds
-            );
+                if (is_timestamp_newer(current_timestamp, vm_log_timestamp) && check == 4) {
+                    vm_log_timestamp = current_timestamp;
+                    msg = line.substr(line_pos, line.size() - line_pos);
 
-            if (is_timestamp_newer(current_timestamp, vm_log_timestamp)) {
-                vm_log_timestamp = current_timestamp;
-                msg = line.substr(line_pos, line.size() - line_pos);
+                    sanitize_format(msg);
 
-				sanitize_format(msg);
-
-                vboxlog_msg(msg.c_str());
+                    vboxlog_msg(msg.c_str());
+                }
             }
         }
-
-        eol_prev_pos = eol_pos + 1;
-        eol_pos = local_vm_log.find("\n", eol_prev_pos);
     }
 }
 
 int VBOX_BASE::dump_screenshot() {
     int    retval;
-    char   screenshot_md5[32];
+    char   screenshot_md5[33];
     double nbytes;
     char*  buf = NULL;
     size_t n;
     FILE*  f = NULL;
     string screenshot_encoded;
     string virtual_machine_slot_directory;
-	string screenshot_location;
+    string screenshot_location;
 
-	get_slot_directory(virtual_machine_slot_directory);
+    get_slot_directory(virtual_machine_slot_directory);
 
-	screenshot_location = virtual_machine_slot_directory;
-	screenshot_location += "/";
-	screenshot_location += SCREENSHOT_FILENAME;
+    screenshot_location = virtual_machine_slot_directory;
+    screenshot_location += "/";
+    screenshot_location += SCREENSHOT_FILENAME;
 
     if (boinc_file_exists(screenshot_location.c_str())) {
 
@@ -350,15 +549,15 @@ int VBOX_BASE::dump_screenshot() {
         free(buf);
 
         fprintf(
-            stderr,
-            "\n"
-            "Screen Shot Information (Base64 Encoded PNG):\n"
-            "MD5 Signature: %s\n"
-            "Data: %s\n"
-            "\n",
-            screenshot_md5,
-            screenshot_encoded.c_str()
-        );
+                stderr,
+                "\n"
+                "Screen Shot Information (Base64 Encoded PNG):\n"
+                "MD5 Signature: %s\n"
+                "Data: %s\n"
+                "\n",
+                screenshot_md5,
+                screenshot_encoded.c_str()
+               );
     }
 
     return 0;
@@ -638,11 +837,11 @@ void VBOX_BASE::sanitize_format(std::string& output) {
     string::iterator iter = output.begin();
     while (iter != output.end()) {
         if (*iter == '%') {
-			// If we find '%', insert an additional '%' so that the we end up with
-			// "%%" in its place.  This with cause printf() type functions to print
-			// % within the formatted output.
-			//
-			iter = output.insert(iter+1, '%');
+            // If we find '%', insert an additional '%' so that the we end up with
+            // "%%" in its place.  This with cause printf() type functions to print
+            // % within the formatted output.
+            //
+            iter = output.insert(iter+1, '%');
             ++iter;
         } else {
             ++iter;
@@ -711,7 +910,7 @@ int VBOX_BASE::launch_vboxsvc() {
                 hVboxSvc = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, p.id);
                 if (hVboxSvc) break;
             }
-            
+
             if (pidVboxSvc && hVboxSvc) {
                 vboxlog_msg("Status Report: Detected vboxsvc.exe. (PID = '%d')", pidVboxSvc);
                 vboxsvc_pid = pidVboxSvc;
@@ -727,17 +926,17 @@ int VBOX_BASE::launch_vboxsvc() {
                 command = "\"VBoxSVC.exe\" --logrotate 1";
 
                 CreateProcess(
-                    NULL,
-                    (LPTSTR)command.c_str(),
-                    NULL,
-                    NULL,
-                    TRUE,
-                    CREATE_NO_WINDOW,
-                    NULL,
-                    (LPTSTR)virtualbox_home_directory.c_str(),
-                    &si,
-                    &pi
-                );
+                        NULL,
+                        (LPTSTR)command.c_str(),
+                        NULL,
+                        NULL,
+                        TRUE,
+                        CREATE_NO_WINDOW,
+                        NULL,
+                        (LPTSTR)virtualbox_home_directory.c_str(),
+                        &si,
+                        &pi
+                        );
 
                 if (pi.hThread) CloseHandle(pi.hThread);
                 if (pi.hProcess) {
@@ -837,27 +1036,30 @@ int VBOX_BASE::launch_vboxvm() {
 
     // Execute command
     if (!CreateProcess(
-        NULL, 
-        cmdline,
-        NULL,
-        NULL,
-        TRUE,
-        CREATE_NO_WINDOW,
-        NULL,
-        NULL,
-        &si,
-        &pi
-    )) {
+
+                NULL, 
+                cmdline,
+                NULL,
+                NULL,
+                TRUE,
+                CREATE_NO_WINDOW,
+                NULL,
+                NULL,
+                &si,
+                &pi
+
+                )) {
+
         vboxlog_msg(
-            "Status Report: Launching virtualbox.exe/vboxheadless.exe failed!."
-        );
+                "Status Report: Launching virtualbox.exe/vboxheadless.exe failed!."
+                );
         vboxlog_msg(
-            "        Error: %s (%d)",
-            windows_format_error_string(GetLastError(), buf, sizeof(buf)),
-            GetLastError()
-        );
+                "        Error: %s (%d)",
+                windows_format_error_string(GetLastError(), buf, sizeof(buf)),
+                GetLastError()
+                );
         goto CLEANUP;
-    } 
+    }
 
     while(1) {
         GetExitCodeProcess(pi.hProcess, &ulExitCode);
@@ -886,16 +1088,16 @@ int VBOX_BASE::launch_vboxvm() {
     if (ulExitCode != STILL_ACTIVE) {
         sanitize_output(output);
         vboxlog_msg(
-            "Status Report: Virtualbox.exe/Vboxheadless.exe exited prematurely!."
-        );
+                "Status Report: Virtualbox.exe/Vboxheadless.exe exited prematurely!."
+                );
         vboxlog_msg(
-            "    Exit Code: %d",
-            ulExitCode
-        );
+                "    Exit Code: %d",
+                ulExitCode
+                );
         vboxlog_msg(
-            "       Output: %s",
-            output.c_str()
-        );
+                "       Output: %s",
+                output.c_str()
+                );
     }
 
     if (pi.hProcess && (ulExitCode == STILL_ACTIVE)) {
@@ -913,13 +1115,13 @@ CLEANUP:
     int pid = fork();
     if (-1 == pid) {
         vboxlog_msg(
-            "Status Report: Launching virtualbox/vboxheadless failed!."
-        );
+                "Status Report: Launching virtualbox/vboxheadless failed!."
+                );
         vboxlog_msg(
-            "        Error: %s (%d)",
-            strerror(errno),
-            errno
-        );
+                "        Error: %s (%d)",
+                strerror(errno),
+                errno
+                );
         retval = ERR_FORK;
     } else if (0 == pid) {
         if (-1 == execv(argv[0], argv)) {
@@ -949,7 +1151,7 @@ int VBOX_BASE::vbm_popen(string& command, string& output, const char* item, bool
     command = "VBoxManage -q " + command;
 
     do {
-        retval = vbm_popen_raw(command, output, timeout);
+        retval = vbm_popen_raw(command, output , timeout);
         sanitize_output(command);
         sanitize_output(output);
 
@@ -978,7 +1180,7 @@ int VBOX_BASE::vbm_popen(string& command, string& output, const char* item, bool
             // lock is held by a previous instance of vboxmanage whos instance data hasn't been
             // cleaned up within vboxsvc yet.
             //
-            // Error Code: VBOX_E_INVALID_OBJECT_STATE (0x80bb0007) 
+            // Error Code: VBOX_E_INVALID_OBJECT_STATE (0x80bb0007)
             //
             if (VBOX_E_INVALID_OBJECT_STATE == (unsigned int)retval) {
                 if (retry_notes.find("Another VirtualBox management") == string::npos) {
@@ -999,7 +1201,7 @@ int VBOX_BASE::vbm_popen(string& command, string& output, const char* item, bool
             // Experiments performed by jujube suggest changing the sleep interval to an exponential
             // style backoff would increase our chances of success.
             //
-            // Error Code: CO_E_SERVER_EXEC_FAILURE (0x80080005) 
+            // Error Code: CO_E_SERVER_EXEC_FAILURE (0x80080005)
             //
             if (CO_E_SERVER_EXEC_FAILURE == (unsigned int)retval) {
                 if (retry_notes.find("Unable to communicate with VirtualBox") == string::npos) {
@@ -1010,12 +1212,13 @@ int VBOX_BASE::vbm_popen(string& command, string& output, const char* item, bool
                     sleep_interval *= 2;
                 }
             }
-            
+
             // Retry?
+
             if (!retry_failures && 
-                (VBOX_E_INVALID_OBJECT_STATE != (unsigned int)retval) && 
-                (CO_E_SERVER_EXEC_FAILURE != (unsigned int)retval)
-            ) {
+                    (VBOX_E_INVALID_OBJECT_STATE != (unsigned int)retval) && 
+                    (CO_E_SERVER_EXEC_FAILURE != (unsigned int)retval)
+               ) {
                 break;
             }
 
@@ -1035,12 +1238,12 @@ int VBOX_BASE::vbm_popen(string& command, string& output, const char* item, bool
             output += "\nNotes:\n\n" + retry_notes;
         }
         vboxlog_msg(
-            "Error in %s for VM: %d\nCommand:\n%s\nOutput:\n%s",
-            item,
-            retval,
-            command.c_str(),
-            output.c_str()
-        );
+                "Error in %s for VM: %d\nCommand:\n%s\nOutput:\n%s",
+                item,
+                retval,
+                command.c_str(),
+                output.c_str()
+                );
     }
 
     return retval;
@@ -1049,13 +1252,13 @@ int VBOX_BASE::vbm_popen(string& command, string& output, const char* item, bool
 // Execute the vbox manage application and copy the output to the buffer.
 //
 int VBOX_BASE::vbm_popen_raw(
-    string& command, string& output,
+        string& command, string& output ,
 #ifdef _WIN32
-    unsigned int timeout
+        unsigned int timeout
 #else
-    unsigned int
+        unsigned int
 #endif
-) {
+        ) {
     size_t errcode_start;
     size_t errcode_end;
     string errcode;
@@ -1103,17 +1306,20 @@ int VBOX_BASE::vbm_popen_raw(
 
     // Execute command
     if (!CreateProcess(
-        NULL, 
-        (LPTSTR)command.c_str(),
-        NULL,
-        NULL,
-        TRUE,
-        CREATE_NO_WINDOW,
-        NULL,
-        NULL,
-        &si,
-        &pi
-    )) {
+
+                NULL, 
+
+                (LPTSTR)command.c_str(),
+                NULL,
+                NULL,
+                TRUE,
+                CREATE_NO_WINDOW,
+                NULL,
+                NULL,
+                &si,
+                &pi
+                )) {
+
         vboxlog_msg("CreateProcess failed! (%d).", GetLastError());
         goto CLEANUP;
     }
@@ -1190,7 +1396,7 @@ CLEANUP:
 
     // Execute command
     fp = popen(modified_command.c_str(), "r");
-    if (fp == NULL){
+    if (fp == NULL) {
         vboxlog_msg("vbm_popen popen failed! (%d).", errno);
         retval = ERR_FOPEN;
     } else {
@@ -1252,18 +1458,18 @@ void VBOX_BASE::vbm_trace(std::string& command, std::string& output, int retval)
     FILE* f = fopen(TRACELOG_FILENAME, "a");
     if (f) {
         fprintf(
-            f,
-            "%s (%d): ",
-            buf,
-            pid
-        );
+                f,
+                "%s (%d): ",
+                buf,
+                pid
+               );
         fprintf(
-            f,
-            "\nCommand: %s\nExit Code: %d\nOutput:\n%s\n",
-            command.c_str(),
-            retval,
-            output.c_str()
-        );
+                f,
+                "\nCommand: %s\nExit Code: %d\nOutput:\n%s\n",
+                command.c_str(),
+                retval,
+                output.c_str()
+               );
         fclose(f);
     }
 }
@@ -1273,3 +1479,4 @@ VBOX_VM::VBOX_VM() {
 
 VBOX_VM::~VBOX_VM() {
 }
+
